package com.xy.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.entity.DeviceRecords;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceRecordsMapper;
import com.xy.utils.DataTime;
import com.xy.utils.JobUtils;
import com.xy.utils.SysDictUtils;
import com.xy.utils.enums.DictEnum;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * <p>
 * 设备操作变更记录job
 * </p>
 *
 * @author lijin
 * @since 2023-01-11
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceRecordsJob {

    private DeviceRecordsMapper deviceRecordsMapper;

    /**
     * 数据清除
     *
     * @return
     */
    @XxlJob("delDeviceRecords")
    public ReturnT<String> delDeviceRecords() {
        //数据保留的天数
        SysDictRedis sysDictRedis = SysDictUtils.get(DictEnum.DATA_CLEAR_SIZE.getKey(), "device_records_day");
        Integer jobParam = JobUtils.getPrams().gtInt();
        Integer saveDay = jobParam != null ? jobParam
                : sysDictRedis != null ? Integer.valueOf(sysDictRedis.getValue())
                : 30;
        LambdaQueryWrapper<DeviceRecords> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceRecords>()
                .le(DeviceRecords::getCreateTime, DataTime.getStringAround(0, 0, (~(saveDay - 1)), 0, 0, 0));
        int delete = deviceRecordsMapper.delete(lambdaQueryWrapper);
        log.info("设备操作变更记录数据清除完成，共{}条", delete);
        return ReturnT.SUCCESS;
    }
}
