package com.xy.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.entity.DeviceTempRecords;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceTempRecordsMapper;
import com.xy.utils.DataTime;
import com.xy.utils.JobUtils;
import com.xy.utils.SysDictUtils;
import com.xy.utils.enums.DictEnum;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * <p>
 * 设备温度日志job
 * </p>
 *
 * @author lijin
 * @since 2023-01-11
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceTempRecordsJob {

    private DeviceTempRecordsMapper deviceTempRecordsMapper;

    /**
     * 数据清除
     *
     * @return
     */
    @XxlJob("delDeviceTempRecords")
    public ReturnT<String> delDeviceTempRecords() {
        //数据保留的天数
        SysDictRedis sysDictRedis = SysDictUtils.get(DictEnum.DATA_CLEAR_SIZE.getKey(), "device_temp_records_day");
        Integer jobParam = JobUtils.getPrams().gtInt();
        Integer saveDay = jobParam != null ? jobParam
                : sysDictRedis != null ? Integer.valueOf(sysDictRedis.getValue())
                : 30;
        LambdaQueryWrapper<DeviceTempRecords> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceTempRecords>()
                .le(DeviceTempRecords::getCreateTime, DataTime.getStringAround(0, 0, (~(saveDay - 1)), 0, 0, 0));
        int delete = deviceTempRecordsMapper.delete(lambdaQueryWrapper);
        log.info("设备温度日志数据清除完成，共{}条", delete);
        return ReturnT.SUCCESS;
    }
}
