package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.DeviceErrorsRecordDto;
import com.xy.entity.DeviceErrorsRecord;
import com.xy.mapper.DeviceErrorsRecordMapper;
import com.xy.utils.Emptys;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 机器-故障表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-09
 */
@Service
@AllArgsConstructor
@Api(tags = "机器-故障表")
public class DeviceErrorsRecordServiceImpl extends ServiceImpl<DeviceErrorsRecordMapper, DeviceErrorsRecord> implements DeviceErrorsRecordService {

    @PostMapping("save")
    @ApiOperation("添加")
    public R save(@RequestBody @Validated DeviceErrorsRecordDto.Save save) {
        DeviceErrorsRecord deviceErrorsRecord = copy(DeviceErrorsRecord.class, save);
        save(deviceErrorsRecord);
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    private R<PageBean<DeviceErrorsRecordDto.Vo>> page(@RequestBody DeviceErrorsRecordDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceErrorsRecord> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceErrorsRecord.class)
                .ge(DeviceErrorsRecord::getCreateTime, page.getBeginCreateTime())
                .le(DeviceErrorsRecord::getCreateTime, page.getEndCreateTime())
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), DeviceErrorsRecord::getCreateTime);
        IPage<DeviceErrorsRecord> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(DeviceErrorsRecordDto.Vo.class, iPage));
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    private R<List<DeviceErrorsRecordDto.Vo>> list(@RequestBody DeviceErrorsRecordDto.SelectList selectList) {
        LambdaQueryWrapper<DeviceErrorsRecord> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceErrorsRecord.class)
                .in(DeviceErrorsRecord::getId, selectList.getIds())
                .in(DeviceErrorsRecord::getDeviceId, selectList.getDeviceIds())
                .build();
        List<DeviceErrorsRecord> list = list(lambdaQueryWrapper);
        return R.ok(copy(DeviceErrorsRecordDto.Vo.class, list));
    }

}
