package com.xy.utils.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * 质检类型对应表列枚举
 */
@Getter
@AllArgsConstructor
public enum QualityEnum {

    /**
     * 软件硬系统检查
     */
    CHECK_SYSTEM("check_system", "system_state", "system_state_begin_time", "system_state_end_time"),

    /**
     * 交易测试检查
     */
    CHECK_TRADE("check_trade", "trade_state", "trade_state_begin_time", "trade_state_end_time"),

    /**
     * 门锁检查
     */
    CHECK_LOCKER("check_locker", "locker_state", "locker_state_begin_time", "locker_state_end_time"),

    /**
     * 温度检查
     */
    CHECK_TEMPERATURE("check_temperature", "temperature_state", "temperature_state_begin_time", "temperature_state_end_time"),

    /**
     * 模拟批量测试
     */
    CHECK_IMITATE_TEST("check_imitate_test", "imitate_test_state", "imitate_test_state_begin_time", "imitate_test_state_end_time"),

    /**
     * 参数设置检查
     */
    CHECK_SETTING("check_setting", "setting_state", "setting_state_begin_time", "setting_state_end_time"),

    ;

    /**
     * 质检类型
     */
    private String type;

    /**
     * 质检状态列名
     */
    private String state;

    /**
     * 质检状态开始时间列名
     */
    private String beginTime;

    /**
     * 质检状态结束时间列名
     */
    private String endTime;

    public static QualityEnum getQualityEnum(String type) {
        QualityEnum[] values = QualityEnum.values();
        for (QualityEnum qualityEnum : values) {
            if (qualityEnum.type.equals(type)) {
                return qualityEnum;
            }
        }
        return null;
    }
}
