package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 机器-故障表
 * </p>
 *
 * @author lijin
 * @since 2023-01-09
 */
public class DeviceErrorsRecordDto {

    @Data
    @Accessors(chain = true)
    public static class Page extends Vo {

        @ApiModelProperty(value = "分页对象", required = true)
        private PageBean page;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-起")
        private LocalDate beginCreateTime;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-始")
        private LocalDate endCreateTime;

        public LocalDateTime getBeginCreateTime() {
            return beginCreateTime == null ? null : beginCreateTime.atTime(0, 0, 0);
        }

        public LocalDateTime getEndCreateTime() {
            return endCreateTime == null ? null : endCreateTime.atTime(23, 59, 59);
        }

    }

    @Data
    @Accessors(chain = true)
    public static class Save extends Vo {

        @NotNull(message = "deviceId不能为空")
        @ApiModelProperty(value = "设备编号", required = true)
        private Long deviceId;
    }

    @Data
    @Accessors(chain = true)
    public static class SelectList extends Vo {

        @ApiModelProperty(value = "id")
        private List<Integer> ids;

        @ApiModelProperty(value = "设备编号")
        private List<Long> deviceIds;
    }

    @Data
    @Accessors(chain = true)
    public static class Vo {

        @ApiModelProperty(value = "id")
        private Integer id;

        @ApiModelProperty(value = "设备编号")
        private Long deviceId;

        @ApiModelProperty(value = "商户ID")
        private Long mercId;

        @ApiModelProperty(value = "故障类型")
        private Integer errorType;

        @ApiModelProperty(value = "故障描述")
        private String errorDescript;

        @ApiModelProperty(value = "上位机版本号")
        private String appUpmVersion;

        @ApiModelProperty(value = "下位机版本号")
        private String appDownmVersion;

        @ApiModelProperty(value = "创建时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTime;


    }

}
